import type { RequestHandler } from '@sveltejs/kit';
import { env } from '$env/dynamic/private';
import { FeedbackAddSchema } from './schema';
import { Feedback, User } from '$lib/server/database';
import crypto from 'crypto';
import type { CreationAttributes } from 'sequelize';
import { env as public_env } from '$env/dynamic/public';

export const POST = (async ({ request, url }) => {
	if (env.API_SECRET && url.searchParams.get('secret') !== env.API_SECRET)
		return new Response(null, { status: 401 });

	const parseResult = await FeedbackAddSchema.safeParseAsync(await request.json());
	if (!parseResult.success) {
		return new Response(null, { status: 400 });
	}
	const data = parseResult.data;

	const feedback = {} as { [k: string]: CreationAttributes<Feedback> };
	for (const user of await User.findAll({
		where: { uuid: data.users },
		attributes: ['id', 'uuid']
	})) {
		feedback[user.uuid] = {
			url_hash: crypto.randomBytes(18).toString('hex'),
			event: data.event,
			draft: true,
			user_id: user.id
		};
	}

	await Feedback.bulkCreate(Object.values(feedback));

	return new Response(
		JSON.stringify(
			Object.entries(feedback).reduce(
				(curr, [k, v]) => {
					curr[k] = `${url.protocol}//${url.host}${public_env.PUBLIC_BASE_PATH || ''}/feedback/${
						v.url_hash
					}`;
					return curr;
				},
				{} as { [k: string]: string }
			)
		),
		{ status: 201 }
	);
}) satisfies RequestHandler;
